<?php

declare(strict_types = 1);

namespace Drupal\Tests\entity_visibility_preview_simple_grants\FunctionalJavascript;

use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\node\NodeInterface;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\Tests\entity_visibility_preview\Traits\EntityVisibilityPreviewTestHelperTrait;
use Drupal\Tests\field\Traits\EntityReferenceTestTrait;

/**
 * Provides helper methods for the EVP module's functional javascript tests.
 */
abstract class EntityVisibilityPreviewListFunctionalJavascriptTestBase extends WebDriverTestBase {

  use EntityVisibilityPreviewTestHelperTrait;
  use EntityReferenceTestTrait;

  /**
   * Url to display using test browser.
   *
   * @var string
   */
  protected $testUrlToDisplay = '/node';

  /**
   * The user field name containing segmentation.
   *
   * @var string
   */
  protected $userFieldName = 'test_taxonomy_condition';

  /**
   * The vocabulary ID used for the tests.
   *
   * @var string
   */
  protected $vocabularyId = 'taxonomy_condition_test';

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'bartik';

  /**
   * {@inheritdoc}
   */
  protected $fieldSettings = [
    'enabled_condition_plugins' => [
      'taxonomy_condition' => 'taxonomy_condition',
      'date_range_condition' => 'date_range_condition',
    ],
    'condition_plugins_settings' => [
      'taxonomy_condition' => [
        'bundles' => [
          'taxonomy_condition_test' => 'taxonomy_condition_test',
        ],
        'user_fields' => [
          'test_taxonomy_condition' => 'test_taxonomy_condition',
        ],
      ],
    ],
  ];

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'datetime',
    'entity_visibility_preview',
    'entity_visibility_preview_simple_grants',
    'node',
    'taxonomy',
    'toolbar',
    'views',
  ];

  /**
   * The list of user permissions.
   *
   * @var array
   */
  protected $userPermissions = [
    'access toolbar',
    'entity_visibility_preview_use_preview',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    // Create article node type.
    if ($this->profile != 'standard') {
      $this->drupalCreateContentType([
        'type' => 'article',
        'name' => 'Article',
      ]);
    }

    $this->createFieldStorage();
    $this->createFieldConfig();

    $taxonomy = Vocabulary::create([
      'name' => $this->randomString(),
      'vid' => $this->vocabularyId,
    ]);
    $taxonomy->save();

    $selection_handler_settings = [
      'target_bundles' => [
        $this->vocabularyId => $this->vocabularyId,
      ],
    ];
    $this->createEntityReferenceField('user', 'user', $this->userFieldName, $this->randomString(), 'taxonomy_term', 'default', $selection_handler_settings, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);

    /** @var \Drupal\Core\Datetime\DateFormatterInterface $date_formatter */
    $date_formatter = $this->container->get('date.formatter');
    $five_days_in_the_past = $date_formatter->format(strtotime('-5 days'), 'custom', DateTimeItemInterface::DATE_STORAGE_FORMAT);
    $five_days_in_the_future = $date_formatter->format(strtotime('+5 days'), 'custom', DateTimeItemInterface::DATE_STORAGE_FORMAT);
    $fifteen_days_in_the_future = $date_formatter->format(strtotime('+15 days'), 'custom', DateTimeItemInterface::DATE_STORAGE_FORMAT);

    $taxonomy_term_1 = Term::create([
      'name' => 'Segment 1',
      'vid' => $this->vocabularyId,
      'uuid' => 'segment1_uuid',
    ]);
    $taxonomy_term_1->save();

    $taxonomy_term_2 = Term::create([
      'name' => 'Segment 2',
      'vid' => $this->vocabularyId,
      'uuid' => 'segment2_uuid',
    ]);
    $taxonomy_term_2->save();

    $nodes = [
      [
        'type' => 'article',
        'title' => 'Article 1 - Segment: 1 - Date: any',
        'status' => NodeInterface::PUBLISHED,
        'field_entity_visibility_preview' => serialize([
          'taxonomy_condition' => [
            'taxonomy_term_uuids' => [$taxonomy_term_1->uuid()],
          ],
        ]),
      ],
      [
        'type' => 'article',
        'title' => 'Article 2 - Segment: 2 - Date: any',
        'status' => NodeInterface::PUBLISHED,
        'field_entity_visibility_preview' => serialize([
          'taxonomy_condition' => [
            'taxonomy_term_uuids' => [$taxonomy_term_2->uuid()],
          ],
        ]),
      ],
      [
        'type' => 'article',
        'title' => 'Article 3 - Segment: 1 - Date: -5 days to +5 days',
        'status' => NodeInterface::PUBLISHED,
        'field_entity_visibility_preview' => serialize([
          'taxonomy_condition' => [
            'taxonomy_term_uuids' => [$taxonomy_term_1->uuid()],
          ],
          'date_range_condition' => [
            'value' => $five_days_in_the_past,
            'end_value' => $five_days_in_the_future,
          ],
        ]),
      ],
      [
        'type' => 'article',
        'title' => 'Article 4 - Segment: 1 - Date: +5 days to +15 days',
        'status' => NodeInterface::PUBLISHED,
        'field_entity_visibility_preview' => serialize([
          'taxonomy_condition' => [
            'taxonomy_term_uuids' => [$taxonomy_term_1->uuid()],
          ],
          'date_range_condition' => [
            'value' => $five_days_in_the_future,
            'end_value' => $fifteen_days_in_the_future,
          ],
        ]),
      ],
      [
        'type' => 'article',
        'title' => 'Article 5 - Segment: 2 - Date: -5 days to +5 days',
        'status' => NodeInterface::PUBLISHED,
        'field_entity_visibility_preview' => serialize([
          'taxonomy_condition' => [
            'taxonomy_term_uuids' => [$taxonomy_term_2->uuid()],
          ],
          'date_range_condition' => [
            'value' => $five_days_in_the_past,
            'end_value' => $five_days_in_the_future,
          ],
        ]),
      ],
      [
        'type' => 'article',
        'title' => 'Article 6 - Segment: 2 - Date: +5 days to +15 days',
        'status' => NodeInterface::PUBLISHED,
        'field_entity_visibility_preview' => serialize([
          'taxonomy_condition' => [
            'taxonomy_term_uuids' => [$taxonomy_term_2->uuid()],
          ],
          'date_range_condition' => [
            'value' => $five_days_in_the_future,
            'end_value' => $fifteen_days_in_the_future,
          ],
        ]),
      ],
      [
        'type' => 'article',
        'title' => 'Article 7 - No evp',
        'status' => NodeInterface::PUBLISHED,
      ],
    ];

    foreach ($nodes as $node) {
      $this->drupalCreateNode($node);
    }
    node_access_rebuild();

    $this->user = $this->drupalCreateUser(
      $this->userPermissions,
      $this->randomString(),
      FALSE,
      [
        'test_taxonomy_condition' => [
          [
            'target_id' => $taxonomy_term_1->id(),
          ],
        ],
      ]
    );
  }

}
