<?php

declare(strict_types = 1);

namespace Drupal\Tests\entity_visibility_preview\Functional;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\node\NodeInterface;

/**
 * Tests for the website preview form.
 *
 * @group entity_visibility_preview
 */
class WebsitePreviewFormTest extends EntityVisibilityPreviewFunctionalTestBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'datetime',
  ];

  /**
   * {@inheritdoc}
   */
  protected $userPermissions = [
    'access content',
    'entity_visibility_preview_use_preview',
  ];

  /**
   * {@inheritdoc}
   */
  protected $fieldSettings = [
    'enabled_condition_plugins' => [
      'date_range_condition' => 'date_range_condition',
    ],
  ];

  /**
   * Test the website preview form with the date range plugin.
   */
  public function testWithDateRangePlugin() {
    /** @var \Drupal\Core\Datetime\DateFormatterInterface $date_formatter */
    $date_formatter = $this->container->get('date.formatter');
    $ten_days_in_the_past = $date_formatter->format(strtotime('-10 days'), 'custom', DateTimeItemInterface::DATE_STORAGE_FORMAT);
    $five_days_in_the_past = $date_formatter->format(strtotime('-5 days'), 'custom', DateTimeItemInterface::DATE_STORAGE_FORMAT);
    $today = $date_formatter->format(strtotime('today'), 'custom', DateTimeItemInterface::DATE_STORAGE_FORMAT);
    $five_days_in_the_future = $date_formatter->format(strtotime('+5 days'), 'custom', DateTimeItemInterface::DATE_STORAGE_FORMAT);
    $ten_days_in_the_future = $date_formatter->format(strtotime('+10 days'), 'custom', DateTimeItemInterface::DATE_STORAGE_FORMAT);

    $node = $this->drupalCreateNode([
      'type' => 'article',
      'status' => NodeInterface::PUBLISHED,
      'field_entity_visibility_preview' => serialize([
        'date_range_condition' => [
          'value' => $five_days_in_the_past,
          'end_value' => $five_days_in_the_future,
        ],
      ]),
    ]);

    $this->drupalLogin($this->user);

    $this->datePreviewTest($node, 200, '', FALSE);
    $this->datePreviewTest($node, 403, $ten_days_in_the_past, FALSE);
    $this->datePreviewTest($node, 200, $five_days_in_the_past, FALSE);
    $this->datePreviewTest($node, 200, $today, FALSE);
    $this->datePreviewTest($node, 403, $five_days_in_the_future, FALSE);
    $this->datePreviewTest($node, 403, $ten_days_in_the_future, FALSE);
    $this->datePreviewTest($node, 200, '', TRUE);
  }

  /**
   * Helper function.
   *
   * Check status code depending on preview date.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node used for tests.
   * @param int $expected_status_code
   *   The expected status code.
   * @param string $preview_date
   *   The preview date.
   * @param bool $reset
   *   If the preview data should be resetted.
   *
   * @SuppressWarnings(PHPMD.BooleanArgumentFlag)
   */
  protected function datePreviewTest(NodeInterface $node, int $expected_status_code = 200, string $preview_date = '', bool $reset = FALSE) {
    if (!empty($preview_date)) {
      $this->drupalPostForm('/admin/content/entity_visibility_preview/preview_form', [
        'preview_date_display' => $preview_date,
      ], $this->t('Update'));
    }
    elseif ($reset) {
      $this->drupalPostForm('/admin/content/entity_visibility_preview/preview_form', [], $this->t('Reset to current date and normal state'));
    }

    $this->drupalGet($node->toUrl());
    $this->assertSession()->statusCodeEquals($expected_status_code);
  }

}
